@@ -18,20 +18,37 @@
 # Copyright (c) OWASP Foundation. All Rights Reserved.
 
 import warnings
-from typing import Optional, Set
+from typing import Iterable, Optional
 from xml.etree import ElementTree
 
-from . import BaseOutput, SchemaVersion
-from .schema import BaseSchemaVersion, SchemaVersion1Dot0, SchemaVersion1Dot1, SchemaVersion1Dot2, SchemaVersion1Dot3, \
-    SchemaVersion1Dot4
-from ..model import AttachedText, ExternalReference, HashType, IdentifiableAction, LicenseChoice, \
-    OrganizationalEntity, OrganizationalContact, Property, Tool
+from sortedcontainers import SortedSet
+
+from ..model import (
+    AttachedText,
+    ExternalReference,
+    HashType,
+    IdentifiableAction,
+    LicenseChoice,
+    OrganizationalContact,
+    OrganizationalEntity,
+    Property,
+    Tool,
+)
 from ..model.bom import Bom
 from ..model.bom_ref import BomRef
 from ..model.component import Component, Patch
 from ..model.release_note import ReleaseNotes
 from ..model.service import Service
-from ..model.vulnerability import Vulnerability, VulnerabilityRating, VulnerabilitySource, BomTargetVersionRange
+from ..model.vulnerability import BomTargetVersionRange, Vulnerability, VulnerabilityRating, VulnerabilitySource
+from . import BaseOutput, SchemaVersion
+from .schema import (
+    BaseSchemaVersion,
+    SchemaVersion1Dot0,
+    SchemaVersion1Dot1,
+    SchemaVersion1Dot2,
+    SchemaVersion1Dot3,
+    SchemaVersion1Dot4,
+)
 
 
 class Xml(BaseOutput, BaseSchemaVersion):
@@ -52,14 +69,17 @@ class Xml(BaseOutput, BaseSchemaVersion):
         elif self.generated:
             return
 
+        bom = self.get_bom()
+        bom.validate()
+
         if self.bom_supports_metadata():
             self._add_metadata_element()
 
-        components_element = ElementTree.SubElement(self._root_bom_element, 'components')
-
         has_vulnerabilities: bool = False
-        if self.get_bom().components:
-            for component in self.get_bom().components:
+
+        components_element = ElementTree.SubElement(self._root_bom_element, 'components')
+        if bom.components:
+            for component in bom.components:
                 component_element = self._add_component_element(component=component)
                 components_element.append(component_element)
                 if self.bom_supports_vulnerabilities_via_extension() and component.has_vulnerabilities():
@@ -79,22 +99,35 @@ class Xml(BaseOutput, BaseSchemaVersion):
                 elif component.has_vulnerabilities():
                     has_vulnerabilities = True
 
-        if self.bom_supports_services():
-            if self.get_bom().services:
-                services_element = ElementTree.SubElement(self._root_bom_element, 'services')
-                for service in self.get_bom().services:
-                    services_element.append(self._add_service_element(service=service))
+        if self.bom_supports_services() and bom.services:
+            services_element = ElementTree.SubElement(self._root_bom_element, 'services')
+            for service in bom.services:
+                services_element.append(self._add_service_element(service=service))
 
-        if self.bom_supports_external_references():
-            if self.get_bom().external_references:
-                self._add_external_references_to_element(
-                    ext_refs=self.get_bom().external_references,
-                    element=self._root_bom_element
-                )
+        if self.bom_supports_external_references() and bom.external_references:
+            self._add_external_references_to_element(
+                ext_refs=bom.external_references,
+                element=self._root_bom_element
+            )
+
+        if self.bom_supports_dependencies() and (bom.metadata.component or bom.components):
+            dep_components: Iterable[Component] = bom.components
+            if bom.metadata.component:
+                dep_components = [bom.metadata.component, *dep_components]
+            dependencies_element = ElementTree.SubElement(self._root_bom_element, 'dependencies')
+            for component in dep_components:
+                dependency_element = ElementTree.SubElement(dependencies_element, 'dependency', {
+                    'ref': str(component.bom_ref)
+                })
+                for dependency in component.dependencies:
+                    ElementTree.SubElement(dependency_element, 'dependency', {
+                        'ref': str(dependency)
+                    })
+            del dep_components
 
         if self.bom_supports_vulnerabilities() and has_vulnerabilities:
             vulnerabilities_element = ElementTree.SubElement(self._root_bom_element, 'vulnerabilities')
-            for component in self.get_bom().components:
+            for component in bom.components:
                 for vulnerability in component.get_vulnerabilities():
                     vulnerabilities_element.append(
                         self._get_vulnerability_as_xml_element_post_1_4(vulnerability=vulnerability)
@@ -111,13 +144,14 @@ class Xml(BaseOutput, BaseSchemaVersion):
 
     # Builder Methods
     def _create_bom_element(self) -> ElementTree.Element:
+        bom = self.get_bom()
         root_attributes = {
             'xmlns': self.get_target_namespace(),
             'version': '1',
-            'serialNumber': self.get_bom().get_urn_uuid()
+            'serialNumber': bom.get_urn_uuid()
         }
 
-        if self.bom_supports_vulnerabilities_via_extension() and self.get_bom().has_vulnerabilities():
+        if self.bom_supports_vulnerabilities_via_extension() and bom.has_vulnerabilities():
             root_attributes['xmlns:v'] = Xml.VULNERABILITY_EXTENSION_NAMESPACE
             ElementTree.register_namespace('v', Xml.VULNERABILITY_EXTENSION_NAMESPACE)
 
@@ -339,7 +373,8 @@ class Xml(BaseOutput, BaseSchemaVersion):
 
         return component_element
 
-    def _add_licenses_to_element(self, licenses: Set[LicenseChoice], parent_element: ElementTree.Element) -> bool:
+    def _add_licenses_to_element(self, licenses: "SortedSet[LicenseChoice]",
+                                 parent_element: ElementTree.Element) -> bool:
         license_output = False
         for license_ in licenses:
             if license_.license:
@@ -442,7 +477,7 @@ class Xml(BaseOutput, BaseSchemaVersion):
         return patch_element
 
     @staticmethod
-    def _add_properties_element(properties: Set[Property], parent_element: ElementTree.Element) -> None:
+    def _add_properties_element(properties: "SortedSet[Property]", parent_element: ElementTree.Element) -> None:
         properties_e = ElementTree.SubElement(parent_element, 'properties')
         for property_ in properties:
             ElementTree.SubElement(
@@ -653,6 +688,10 @@ class Xml(BaseOutput, BaseSchemaVersion):
                     for version in target.versions:
                         Xml._add_bom_target_version_range(parent_element=v_target_versions_element, version=version)
 
+        # properties
+        if vulnerability.properties:
+            Xml._add_properties_element(properties=vulnerability.properties, parent_element=vulnerability_element)
+
         return vulnerability_element
 
     @staticmethod
@@ -720,7 +759,7 @@ class Xml(BaseOutput, BaseSchemaVersion):
 
         return vulnerability_element
 
-    def _add_external_references_to_element(self, ext_refs: Set[ExternalReference],
+    def _add_external_references_to_element(self, ext_refs: "SortedSet[ExternalReference]",
                                             element: ElementTree.Element) -> None:
         ext_refs_element = ElementTree.SubElement(element, 'externalReferences')
         for external_reference in ext_refs:
@@ -745,7 +784,7 @@ class Xml(BaseOutput, BaseSchemaVersion):
         return at_element
 
     @staticmethod
-    def _add_hashes_to_element(hashes: Set[HashType], element: ElementTree.Element) -> None:
+    def _add_hashes_to_element(hashes: "SortedSet[HashType]", element: ElementTree.Element) -> None:
         hashes_e = ElementTree.SubElement(element, 'hashes')
         for h in hashes:
             ElementTree.SubElement(
